@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,14 +12,16 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "idsuggestions.h"
 
+#include <QRegularExpression>
+
 #include <KSharedConfig>
 #include <KConfigGroup>
-#include <KLocale>
+#include <KLocalizedString>
 
 #include "journalabbreviations.h"
 #include "encoderlatex.h"
@@ -34,29 +36,43 @@ private:
 public:
 
     IdSuggestionsPrivate(IdSuggestions *parent)
-            : p(parent), config(KSharedConfig::openConfig(QLatin1String("kbibtexrc"))), group(config, IdSuggestions::configGroupName) {
+            : p(parent), config(KSharedConfig::openConfig(QStringLiteral("kbibtexrc"))), group(config, IdSuggestions::configGroupName) {
         // nothing
     }
 
     QString normalizeText(const QString &input) const {
         static const QRegExp unwantedChars = QRegExp("[^-_:/=+a-zA-Z0-9]+");
-        return EncoderLaTeX::instance()->convertToPlainAscii(input).remove(unwantedChars);
+        return EncoderLaTeX::instance().convertToPlainAscii(input).remove(unwantedChars);
     }
 
-    inline int extractYear(const Entry &entry) const {
+    int numberFromEntry(const Entry &entry, const QString &field) const {
+        static const QRegularExpression firstDigits(QStringLiteral("^[0-9]+"));
+        const QString text = PlainTextValue::text(entry.value(field));
+        const QRegularExpressionMatch match = firstDigits.match(text);
+        if (!match.hasMatch()) return -1;
+
         bool ok = false;
-        int result = PlainTextValue::text(entry.value(Entry::ftYear)).toInt(&ok);
+        const int result = match.captured(0).toInt(&ok);
         return ok ? result : -1;
     }
 
+    QString pageNumberFromEntry(const Entry &entry) const {
+        static const QRegularExpression whitespace(QStringLiteral("[ \t]+"));
+        static const QRegularExpression pageNumber(QStringLiteral("[a-z0-9+:]+"), QRegularExpression::CaseInsensitiveOption);
+        const QString text = PlainTextValue::text(entry.value(Entry::ftPages)).remove(whitespace).remove(QStringLiteral("mbox"));
+        const QRegularExpressionMatch match = pageNumber.match(text);
+        if (!match.hasMatch()) return QString();
+        return match.captured(0);
+    }
+
     QString translateTitleToken(const Entry &entry, const struct IdSuggestionTokenInfo &tti, bool removeSmallWords) const {
         /// list of small words taken from OCLC:
         /// http://www.oclc.org/developer/develop/web-services/worldcat-search-api/bibliographic-resource.en.html
-        static const QStringList smallWords = i18nc("Small words that can be removed from titles when generating id suggestions; separated by pipe symbol", "a|als|am|an|are|as|at|auf|aus|be|but|by|das|dass|de|der|des|dich|dir|du|er|es|for|from|had|have|he|her|his|how|ihr|ihre|ihres|im|in|is|ist|it|kein|la|le|les|mein|mich|mir|mit|of|on|sein|sie|that|the|this|to|un|une|von|was|wer|which|wie|wird|with|yousie|that|the|this|to|un|une|von|was|wer|which|wie|wird|with|you").split(QLatin1String("|"), QString::SkipEmptyParts);
+        static const QStringList smallWords = i18nc("Small words that can be removed from titles when generating id suggestions; separated by pipe symbol", "a|als|am|an|are|as|at|auf|aus|be|but|by|das|dass|de|der|des|dich|dir|du|er|es|for|from|had|have|he|her|his|how|ihr|ihre|ihres|im|in|is|ist|it|kein|la|le|les|mein|mich|mir|mit|of|on|sein|sie|that|the|this|to|un|une|von|was|wer|which|wie|wird|with|yousie|that|the|this|to|un|une|von|was|wer|which|wie|wird|with|you").split(QStringLiteral("|"), QString::SkipEmptyParts);
 
         QString result;
         bool first = true;
-        static const QRegExp sequenceOfSpaces(QLatin1String("\\s+"));
+        static const QRegExp sequenceOfSpaces(QStringLiteral("\\s+"));
         const QStringList titleWords = PlainTextValue::text(entry.value(Entry::ftTitle)).split(sequenceOfSpaces, QString::SkipEmptyParts);
         int index = 0;
         for (QStringList::ConstIterator it = titleWords.begin(); it != titleWords.end(); ++it, ++index) {
@@ -107,14 +123,15 @@ public:
             /// Check if camel case is requests
             if (ati.caseChange == IdSuggestions::ccToCamelCase) {
                 /// Get components of the author's last name
-                const QStringList nameComponents = author.split(QLatin1String(" "), QString::SkipEmptyParts);
+                const QStringList nameComponents = author.split(QStringLiteral(" "), QString::SkipEmptyParts);
                 QStringList newNameComponents;
+                newNameComponents.reserve(nameComponents.size());
                 /// Camel-case each name component
-                foreach (const QString &nameComponent, nameComponents) {
+                for (const QString &nameComponent : nameComponents) {
                     newNameComponents.append(nameComponent[0].toUpper() + nameComponent.mid(1));
                 }
                 /// Re-assemble name from camel-cased components
-                author = newNameComponents.join(QLatin1String(" "));
+                author = newNameComponents.join(QStringLiteral(" "));
             }
             if (
                 (index >= ati.startWord && index <= ati.endWord) ///< check for requested author range
@@ -146,14 +163,14 @@ public:
     }
 
     QString translateJournalToken(const Entry &entry, const struct IdSuggestionTokenInfo &jti) const {
-        static const QRegExp sequenceOfSpaces(QLatin1String("\\s+"));
+        static const QRegExp sequenceOfSpaces(QStringLiteral("\\s+"));
         QString journalName = PlainTextValue::text(entry.value(Entry::ftJournal));
         journalName = JournalAbbreviations::self()->toShortName(journalName);
-        QStringList journalWords = journalName.split(sequenceOfSpaces, QString::SkipEmptyParts);
+        const QStringList journalWords = journalName.split(sequenceOfSpaces, QString::SkipEmptyParts);
 
         QString result;
-        for (QStringList::ConstIterator it = journalWords.constBegin(); it != journalWords.constEnd(); ++it) {
-            QString journalComponent = normalizeText(*it);
+        for (const QString &word : journalWords) {
+            QString journalComponent = normalizeText(word);
 
             /// Try to keep sequences of capital letters at the start of the journal name,
             /// those may already be abbreviations.
@@ -207,13 +224,13 @@ public:
             return translateAuthorsToken(entry, ati);
         }
         case 'y': {
-            int year = extractYear(entry);
+            int year = numberFromEntry(entry, Entry::ftYear);
             if (year > -1)
                 return QString::number(year % 100 + 100).mid(1);
             break;
         }
         case 'Y': {
-            int year = extractYear(entry);
+            const int year = numberFromEntry(entry, Entry::ftYear);
             if (year > -1)
                 return QString::number(year % 10000 + 10000).mid(1);
             break;
@@ -229,6 +246,12 @@ public:
             const struct IdSuggestionTokenInfo jti = p->evalToken(token.mid(1));
             return translateJournalToken(entry, jti);
         }
+        case 'v': {
+            return normalizeText(PlainTextValue::text(entry.value(Entry::ftVolume)));
+        }
+        case 'p': {
+            return pageNumberFromEntry(entry);
+        }
         case '"': return token.mid(1);
         }
 
@@ -244,16 +267,16 @@ public:
     }
 };
 
-const QString IdSuggestions::keyDefaultFormatString = QLatin1String("DefaultFormatString");
+const QString IdSuggestions::keyDefaultFormatString = QStringLiteral("DefaultFormatString");
 const QString IdSuggestions::defaultDefaultFormatString = QString();
-const QString IdSuggestions::keyFormatStringList = QLatin1String("FormatStringList");
-const QStringList IdSuggestions::defaultFormatStringList = QStringList() << QLatin1String("A") << QLatin1String("A2|y") << QLatin1String("A3|y") << QLatin1String("A4|y|\":|T5") << QLatin1String("al|\":|T") << QLatin1String("al|y") << QLatin1String("al|Y") << QLatin1String("Al\"-|\"-|y") << QLatin1String("Al\"+|Y") << QLatin1String("al|y|T") << QLatin1String("al|Y|T3") << QLatin1String("al|Y|T3l") << QLatin1String("a|\":|Y|\":|T1") << QLatin1String("a|y") << QLatin1String("A|\":|Y");
-const QString IdSuggestions::configGroupName = QLatin1String("IdSuggestions");
+const QString IdSuggestions::keyFormatStringList = QStringLiteral("FormatStringList");
+const QStringList IdSuggestions::defaultFormatStringList = QStringList() << QStringLiteral("A") << QStringLiteral("A2|y") << QStringLiteral("A3|y") << QStringLiteral("A4|y|\":|T5") << QStringLiteral("al|\":|T") << QStringLiteral("al|y") << QStringLiteral("al|Y") << QStringLiteral("Al\"-|\"-|y") << QStringLiteral("Al\"+|Y") << QStringLiteral("al|y|T") << QStringLiteral("al|Y|T3") << QStringLiteral("al|Y|T3l") << QStringLiteral("a|\":|Y|\":|T1") << QStringLiteral("a|y") << QStringLiteral("A|\":|Y");
+const QString IdSuggestions::configGroupName = QStringLiteral("IdSuggestions");
 
 IdSuggestions::IdSuggestions()
         : d(new IdSuggestionsPrivate(this))
 {
-    // nothing
+    /// nothing
 }
 
 IdSuggestions::~IdSuggestions()
@@ -264,8 +287,8 @@ IdSuggestions::~IdSuggestions()
 QString IdSuggestions::formatId(const Entry &entry, const QString &formatStr) const
 {
     QString id;
-    QStringList tokenList = formatStr.split(QLatin1String("|"), QString::SkipEmptyParts);
-    foreach (const QString &token, tokenList) {
+    const QStringList tokenList = formatStr.split(QStringLiteral("|"), QString::SkipEmptyParts);
+    for (const QString &token : tokenList) {
         id.append(d->translateToken(entry, token));
     }
 
@@ -296,7 +319,8 @@ QStringList IdSuggestions::formatIdList(const Entry &entry) const
 {
     const QStringList formatStrings = d->formatStringList();
     QStringList result;
-    foreach (const QString &formatString, formatStrings) {
+    result.reserve(formatStrings.size());
+    for (const QString &formatString : formatStrings) {
         result << formatId(entry, formatString);
     }
     return result;
@@ -305,8 +329,8 @@ QStringList IdSuggestions::formatIdList(const Entry &entry) const
 QStringList IdSuggestions::formatStrToHuman(const QString &formatStr) const
 {
     QStringList result;
-    QStringList tokenList = formatStr.split(QLatin1String("|"), QString::SkipEmptyParts);
-    foreach (const QString &token, tokenList) {
+    const QStringList tokenList = formatStr.split(QStringLiteral("|"), QString::SkipEmptyParts);
+    for (const QString &token : tokenList) {
         QString text;
         if (token[0] == 'a' || token[0] == 'A' || token[0] == 'z') {
             struct IdSuggestionTokenInfo info = evalToken(token.mid(1));
@@ -388,6 +412,10 @@ QStringList IdSuggestions::formatStrToHuman(const QString &formatStr) const
             case IdSuggestions::ccNoChange:
                 break;
             }
+        } else if (token[0] == 'v') {
+            text.append(i18n("Volume"));
+        } else if (token[0] == 'p') {
+            text.append(i18n("First page number"));
         } else if (token[0] == '"')
             text.append(i18n("Text: '%1'", token.mid(1)));
         else
