@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2004-2014 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
+ *   Copyright (C) 2004-2017 by Thomas Fischer <fischer@unix-ag.uni-kl.de> *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -12,7 +12,7 @@
  *   GNU General Public License for more details.                          *
  *                                                                         *
  *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, see <http://www.gnu.org/licenses/>.  *
+ *   along with this program; if not, see <https://www.gnu.org/licenses/>. *
  ***************************************************************************/
 
 #include "journalabbreviations.h"
@@ -21,9 +21,9 @@
 #include <QFile>
 #include <QTextStream>
 #include <QRegExp>
+#include <QStandardPaths>
 
-#include <KStandardDirs>
-#include <KDebug>
+#include "logging_processing.h"
 
 class JournalAbbreviations::Private
 {
@@ -35,7 +35,7 @@ private:
 
 public:
     Private(JournalAbbreviations */* UNUSED parent*/)
-        : /* UNUSED p(parent),*/ journalFilename(KGlobal::dirs()->findResource("data", QLatin1String("kbibtex/jabref_journalabbrevlist.txt")))
+        : /* UNUSED p(parent),*/ journalFilename(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("kbibtex/jabref_journalabbrevlist.txt")))
     {
         /// nothing
     }
@@ -46,7 +46,7 @@ public:
 
         QFile journalFile(journalFilename);
         if (journalFile.open(QFile::ReadOnly)) {
-            static const QRegExp splitRegExp(QLatin1String("\\s*[=;]\\s*"));
+            static const QRegExp splitRegExp(QStringLiteral("\\s*[=;]\\s*"));
 
             QTextStream ts(&journalFile);
             ts.setCodec("utf8");
@@ -74,7 +74,7 @@ public:
 
             return !leftToRightMap.isEmpty();
         } else {
-            kWarning() << "Cannot open journal abbreviation list file at" << journalFilename;
+            qCWarning(LOG_KBIBTEX_PROCESSING) << "Cannot open journal abbreviation list file at" << journalFilename;
             return false;
         }
     }
@@ -92,7 +92,7 @@ public:
     }
 };
 
-JournalAbbreviations *JournalAbbreviations::instance = NULL;
+JournalAbbreviations *JournalAbbreviations::instance = nullptr;
 
 JournalAbbreviations::JournalAbbreviations()
         : d(new JournalAbbreviations::Private(this))
@@ -106,7 +106,7 @@ JournalAbbreviations::~JournalAbbreviations()
 }
 
 JournalAbbreviations *JournalAbbreviations::self() {
-    if (instance == NULL)
+    if (instance == nullptr)
         instance = new JournalAbbreviations();
     return instance;
 }
